#define SYS_REGPAD5255_OFFSET   0x50/4
#define SYS_REGPAD5659_OFFSET   0x54/4

#define SYS_REGCLKCFG_OFFSET    0x08/4


// ========================================================================== //
//                              R E G I S T E R S                             //
// ========================================================================== //
#define _PWM_CONTROL_OFFSET                0x00    // Control register (PCM only)
#define _PWM_CTRL_OFFSET                   0x01    // Control register
#define _PWM_PRESCALER_OFFSET              0x02    // Prescaler register
#define _PWM_CNT16_LSB_OFFSET              0x03    // Counter LSB register
#define _PWM_CNT16_MSB_OFFSET              0x04    // Counter MSB register
#define _PWM_CMP16_0_LSB_OFFSET            0x05    // Comparator 0 LSB register
#define _PWM_CMP16_0_MSB_OFFSET            0x06    // Comparator 0 MSB register
#define _PWM_CMP16_1_LSB_OFFSET            0x07    // Comparator 1 LSB register
#define _PWM_CMP16_1_MSB_OFFSET            0x08    // Comparator 1 MSB register
#define _PWM_CMP16_2_LSB_OFFSET            0x09    // Comparator 2 LSB register
#define _PWM_CMP16_2_MSB_OFFSET            0x0A    // Comparator 2 MSB register
#define _PWM_CMP16_3_LSB_OFFSET            0x0B    // Comparator 3 LSB register
#define _PWM_CMP16_3_MSB_OFFSET            0x0C    // Comparator 3 MSB register
#define _PWM_CMP16_4_LSB_OFFSET            0x0D    // Comparator 4 LSB register
#define _PWM_CMP16_4_MSB_OFFSET            0x0E    // Comparator 4 MSB register
#define _PWM_CMP16_5_LSB_OFFSET            0x0F    // Comparator 5 LSB register
#define _PWM_CMP16_5_MSB_OFFSET            0x10    // Comparator 5 MSB register
#define _PWM_CMP16_6_LSB_OFFSET            0x11    // Comparator 6 LSB register
#define _PWM_CMP16_6_MSB_OFFSET            0x12    // Comparator 6 MSB register
#define _PWM_CMP16_7_LSB_OFFSET            0x13    // Comparator 7 LSB register
#define _PWM_CMP16_7_MSB_OFFSET            0x14    // Comparator 7 MSB register
#define _PWM_OUT_TOGGLE_EN_0_OFFSET        0x15    // Out toggle enable 0 register
#define _PWM_OUT_TOGGLE_EN_1_OFFSET        0x16    // Out toggle enable 1 register
#define _PWM_OUT_TOGGLE_EN_2_OFFSET        0x17    // Out toggle enable 2 register
#define _PWM_OUT_TOGGLE_EN_3_OFFSET        0x18    // Out toggle enable 3 register
#define _PWM_OUT_TOGGLE_EN_4_OFFSET        0x19    // Out toggle enable 4 register
#define _PWM_OUT_TOGGLE_EN_5_OFFSET        0x1A    // Out toggle enable 5 register
#define _PWM_OUT_TOGGLE_EN_6_OFFSET        0x1B    // Out toggle enable 6 register
#define _PWM_OUT_TOGGLE_EN_7_OFFSET        0x1C    // Out toggle enable 7 register
#define _PWM_OUT_CLR_EN_OFFSET             0x1D    // Out clear enable register
#define _PWM_CTRL_BL_CMP8_OFFSET           0x1E    // PWM Control CMP8 register
#define _PWM_INIT_OFFSET                   0x1F    // Init state register
#define _PWM_INT_MASK_OFFSET               0x20    // Interrupt mask register
#define _PWM_INT_STATUS_OFFSET             0x21    // Interrupt status register
#define _PWM_SAMPLE_FREQ_H_OFFSET          0x22    // Sample rate MSB register
#define _PWM_SAMPLE_FREQ_L_OFFSET          0x23    // Sample rate LSB register
#define _PWM_VOLUME_OFFSET                 0x24    // Volume control register
#define _PWM_FIFO_OFFSET                   0x3C/2    // FIFO access register (16-bit)

// ============================================================================================== //
//                                     R E G I S T E R    B I T S                                                      //
// ============================================================================================== //
// CONTROL bits
#define CONTROL_CH01_AUTO                           0x80
#define CONTROL_CH01_MONO                           0x40
#define CONTROL_CH01_8BIT                           0x20
#define CONTROL_CH01_SCALE                          0x10
#define CONTROL_CH01_FILTER                         0x08
#define CONTROL_PCM_EN                              0x04        // Enable the PCM function
#define CONTROL_DEV_EN                              0x02        // Not in use
#define CONTROL_PWM_SOFT_RESET                      0x01        // Internal soft reset

// CTRL bits
#define CTRL_PWM_INIT_IMASK                         0x10
// Bits [3..2] configure trigger type: 00 = disabled, 01 = positive edge, 10 = negative edge, 11 = any edge
#define CTRL_PWM_TRIGGER_DISABLED                ~(0x03 << 1)
#define CTRL_PWM_TRIGGER_POS_EDGE                (0x01 << 1)
#define CTRL_PWM_TRIGGER_NEG_EDGE                (0x02 << 1)
#define CTRL_PWM_TRIGGER_ANY_EDGE                (0x03 << 1)
#define CTRL_PWM_EN                              0x01        // Enable and start PWM

// INT_MASK and INT_STATUS bits
#define PWM_INT                                         0x01
#define PWM_BUFUNDER_INT                                0x02
#define PWM_BUFOV_INT                                   0x04
#define PWM_BUFHALF_INT                                 0x08
#define PWM_BUFFULL_INT                                 0x10
#define PWM_BUFEMPTY_INT                                0x20

// PWM port definitions
#define _PWM_0                               0
#define _PWM_1                               1
#define _PWM_2                               2
#define _PWM_3                               3
#define _PWM_4                               4
#define _PWM_5                               5
#define _PWM_6                               6
#define _PWM_7                               7

// comparator definitions
#define _COMPARATOR_0                        0
#define _COMPARATOR_1                        1
#define _COMPARATOR_2                        2
#define _COMPARATOR_3                        3
#define _COMPARATOR_4                        4
#define _COMPARATOR_5                        5
#define _COMPARATOR_6                        6
#define _COMPARATOR_7                        7

// PWM mask definitions for use with restore initial state enable IOCTL
#define _MASK_PWM_0                          (1 << 0)
#define _MASK_PWM_1                          (1 << 1)
#define _MASK_PWM_2                          (1 << 2)
#define _MASK_PWM_3                          (1 << 3)
#define _MASK_PWM_4                          (1 << 4)
#define _MASK_PWM_5                          (1 << 5)
#define _MASK_PWM_6                          (1 << 6)
#define _MASK_PWM_7                          (1 << 7)

// comparator mask definitions for use with output toggle enable IOCTL
#define _MASK_COMPARATOR_0                   (1 << 0)
#define _MASK_COMPARATOR_1                   (1 << 1)
#define _MASK_COMPARATOR_2                   (1 << 2)
#define _MASK_COMPARATOR_3                   (1 << 3)
#define _MASK_COMPARATOR_4                   (1 << 4)
#define _MASK_COMPARATOR_5                   (1 << 5)
#define _MASK_COMPARATOR_6                   (1 << 6)
#define _MASK_COMPARATOR_7                   (1 << 7)

// PWM trigger mode definitions
#define _PWM_TRIGGER_MODE_DISABLED             0
#define _PWM_TRIGGER_MODE_POSITIVE_EDGE        1
#define _PWM_TRIGGER_MODE_NEGATIVE_EDGE        2
#define _PWM_TRIGGER_MODE_ANY_EDGE             3

// PCM configuration definitions
#define _PCM_FIFO_ENABLED                      0x80
#define _PCM_FIFO_DISABLED                     0x00
#define _PCM_FIFO_MONO                         0x40
#define _PCM_FIFO_STEREO                       0x00
#define _PCM_8BIT                              0x20
#define _PCM_16BIT                             0x00
#define _PCM_SCALING_ENABLED                   0x10
#define _PCM_SCALING_DISABLED                  0x00
#define _PCM_FILTER_ENABLED                    0x08
#define _PCM_FILTER_DISABLED                   0x00

// Basic configuration structure declaration
typedef struct
{      
        unsigned char  prescaler;
        unsigned short counter;
        unsigned char  initialState;
        unsigned char  restoreInitialStateMask;
        unsigned char  numCycles;
        unsigned char  triggerMode;
} TPWMBasicConfig;



/*--------------------------------------------------------------------------------------------------
 Function used for initializing system registers.

 Parameters:
 None
--------------------------------------------------------------------------------------------------*/
void PWM_Pad_Init(unsigned long *base) {
  // Enable PWM function        (bit 2)
  *(base + SYS_REGCLKCFG_OFFSET) |= 0x00000004;
  // Enable PWM output to pads by setting bits [7:6] to 01
  *(base + SYS_REGPAD5255_OFFSET) = (*(base + SYS_REGPAD5255_OFFSET) & 0x3F3F3F3F) | 0x40404040;
  *(base + SYS_REGPAD5659_OFFSET) = (*(base + SYS_REGPAD5659_OFFSET) & 0x3F3F3F3F) | 0x40404040;
  
}

/*--------------------------------------------------------------------------------------------------
 Function fills FIFO for PCM mode.

 Parameters:
 buffer - pointer to buffer (16-bit data)
 size   - number of 16-bit words to fill
--------------------------------------------------------------------------------------------------*/
void PWM_FillFIFO(unsigned int *buffer, unsigned int size, unsigned char *pwm_base) {
  unsigned int i;
  
  for (i = 0; i < size; i++) {
    *((unsigned int *)pwm_base + _PWM_FIFO_OFFSET) = buffer[i];
  }
}

/*--------------------------------------------------------------------------------------------------
 Function initializes PWM device.

 Parameters:
 conf - pointer TPWMBasicConfig structure which contains settings
--------------------------------------------------------------------------------------------------*/
void PWM_Init(TPWMBasicConfig * conf, unsigned char *pwm_base) {
  unsigned char regVal;
  
  // Set prescaler
  *(pwm_base + _PWM_PRESCALER_OFFSET) = conf->prescaler;
  // Set counter
  regVal = (unsigned char) (conf->counter & 0x00FF);
  *(pwm_base + _PWM_CNT16_LSB_OFFSET) = regVal;
  regVal = (unsigned char) ((conf->counter >> 8) & 0x00FF);
  *(pwm_base + _PWM_CNT16_MSB_OFFSET) = regVal;
  // Set number of cycles
  *(pwm_base + _PWM_CTRL_BL_CMP8_OFFSET) = conf->numCycles;
  // Set initial state
  *(pwm_base + _PWM_INIT_OFFSET) = conf->initialState;
  // Set restore state
  *(pwm_base + _PWM_OUT_CLR_EN_OFFSET) = conf->restoreInitialStateMask;
  // Set trigger mode
  switch(conf->triggerMode) {
    case _PWM_TRIGGER_MODE_DISABLED: {
      regVal = *(pwm_base + _PWM_CTRL_OFFSET);
      regVal &= CTRL_PWM_TRIGGER_DISABLED;
      *(pwm_base + _PWM_CTRL_OFFSET) = regVal;
      break;
    }
    case _PWM_TRIGGER_MODE_POSITIVE_EDGE: {
      regVal = *(pwm_base + _PWM_CTRL_OFFSET);
      regVal &= CTRL_PWM_TRIGGER_DISABLED;
      regVal |= CTRL_PWM_TRIGGER_POS_EDGE;
      *(pwm_base + _PWM_CTRL_OFFSET) = regVal;
      break;
    }
    case _PWM_TRIGGER_MODE_NEGATIVE_EDGE: {
      regVal = *(pwm_base + _PWM_CTRL_OFFSET);
      regVal &= CTRL_PWM_TRIGGER_DISABLED;
      regVal |= CTRL_PWM_TRIGGER_NEG_EDGE;
      *(pwm_base + _PWM_CTRL_OFFSET) = regVal;
      break;
    }
    case _PWM_TRIGGER_MODE_ANY_EDGE: {
      regVal = *(pwm_base + _PWM_CTRL_OFFSET);
      regVal |= CTRL_PWM_TRIGGER_ANY_EDGE;
      *(pwm_base + _PWM_CTRL_OFFSET) = regVal;
      break;
    }
    default: {
      regVal = *(pwm_base + _PWM_CTRL_OFFSET);
      regVal &= CTRL_PWM_TRIGGER_DISABLED;
      *(pwm_base + _PWM_CTRL_OFFSET) = regVal;
      break;
    }
  }
}

/*--------------------------------------------------------------------------------------------------
 Function configures PCM output.

 Parameters:
 configMask - various configuration, see line 145
--------------------------------------------------------------------------------------------------*/
void PWM_ConfigPCM(unsigned char configMask, unsigned char *pwm_base) {
  unsigned char regVal;
  
  regVal = *(pwm_base + _PWM_CONTROL_OFFSET);
  regVal &= 0x07;          // clear the 5 MSB bits, which set the configurations. Keep the 3 LSB bits untouched.
  regVal |= configMask;    // configMask only changes the 5 MSB bits
  *(pwm_base + _PWM_CONTROL_OFFSET) = regVal;
}

/*--------------------------------------------------------------------------------------------------
 Function enables PCM output.

 Parameters:
 None
--------------------------------------------------------------------------------------------------*/
void PWM_EnablePCM(unsigned char *pwm_base) {
  unsigned char regVal;
  regVal = *(pwm_base + _PWM_CONTROL_OFFSET);
  regVal |= CONTROL_PCM_EN;
  *(pwm_base + _PWM_CONTROL_OFFSET) = regVal;
}

/*--------------------------------------------------------------------------------------------------
 Function sets PCM filter sample rate.

 Parameters:
 sampleRate - sample rate in Hz
--------------------------------------------------------------------------------------------------*/
void PWM_SetPCMFilterSampleRate(unsigned int sampleRate, unsigned char *pwm_base) {
  *(pwm_base + _PWM_SAMPLE_FREQ_L_OFFSET) = (unsigned char)sampleRate;
  *(pwm_base + _PWM_SAMPLE_FREQ_H_OFFSET) = (unsigned char)(sampleRate >> 8);
}

/*--------------------------------------------------------------------------------------------------
 Function disables PCM output.

 Parameters:
 None
--------------------------------------------------------------------------------------------------*/
void PWM_DisablePCM(unsigned char *pwm_base) {
  unsigned char regVal;
  
  regVal = *(pwm_base + _PWM_CONTROL_OFFSET);
  regVal &= ~(CONTROL_PCM_EN);
  *(pwm_base + _PWM_CONTROL_OFFSET) = regVal;
}

/*--------------------------------------------------------------------------------------------------
 Function sets a comparator value.

 Parameters:
 comparator - the comparator to set value
 value      - comparator's value
--------------------------------------------------------------------------------------------------*/
void PWM_SetComparatorValue(unsigned char comparator, unsigned int value, unsigned char * pwm_base) {
  unsigned char regVal;
  
  switch(comparator)
  {
    case _COMPARATOR_0: {
      regVal = (unsigned char) value;
      *(pwm_base + _PWM_CMP16_0_LSB_OFFSET) = regVal;
      regVal = (unsigned char) (value >> 8);
      *(pwm_base + _PWM_CMP16_0_MSB_OFFSET) = regVal;
      break;
    }
    case _COMPARATOR_1: {
      regVal = (unsigned char) value;
      *(pwm_base + _PWM_CMP16_1_LSB_OFFSET) = regVal;
      regVal = (unsigned char) (value >> 8);
      *(pwm_base + _PWM_CMP16_1_MSB_OFFSET) = regVal;
      break;
    }
    case _COMPARATOR_2: {
      regVal = (unsigned char) value;
      *(pwm_base + _PWM_CMP16_2_LSB_OFFSET) = regVal;
      regVal = (unsigned char) (value >> 8);
      *(pwm_base + _PWM_CMP16_2_MSB_OFFSET) = regVal;
      break;
    }
    case _COMPARATOR_3: {
      regVal = (unsigned char) value;
      *(pwm_base + _PWM_CMP16_3_LSB_OFFSET) = regVal;
      regVal = (unsigned char) (value >> 8);
      *(pwm_base + _PWM_CMP16_3_MSB_OFFSET) = regVal;
      break;
    }
    case _COMPARATOR_4: {
      regVal = (unsigned char) value;
      *(pwm_base + _PWM_CMP16_4_LSB_OFFSET) = regVal;
      regVal = (unsigned char) (value >> 8);
      *(pwm_base + _PWM_CMP16_4_MSB_OFFSET) = regVal;
      break;
    }
    case _COMPARATOR_5: {
      regVal = (unsigned char) value;
      *(pwm_base + _PWM_CMP16_5_LSB_OFFSET) = regVal;
      regVal = (unsigned char) (value >> 8);
      *(pwm_base + _PWM_CMP16_5_MSB_OFFSET) = regVal;
      break;
    }
    case _COMPARATOR_6: {
      regVal = (unsigned char) value;
      *(pwm_base + _PWM_CMP16_6_LSB_OFFSET) = regVal;
      regVal = (unsigned char) (value >> 8);
      *(pwm_base + _PWM_CMP16_6_MSB_OFFSET) = regVal;
      break;
    }
    case _COMPARATOR_7: {
      regVal = (unsigned char) value;
      *(pwm_base + _PWM_CMP16_7_LSB_OFFSET) = regVal;
      regVal = (unsigned char) (value >> 8);
      *(pwm_base + _PWM_CMP16_7_MSB_OFFSET) = regVal;
      break;
    }
  }
}

/*--------------------------------------------------------------------------------------------------
 Function specifies which comparators a PWM output should toggle on.

 Parameters:
 pwmNum         - the PWM to configure
 comparatorMask - mask to select the comparators to be associated with the PWM
--------------------------------------------------------------------------------------------------*/
void PWM_SetOutputToggleEnables(unsigned char pwmNum, unsigned char comparatorMask, unsigned char *pwm_base) {
  switch(pwmNum) {
    case _PWM_0: {
      *(pwm_base + _PWM_OUT_TOGGLE_EN_0_OFFSET) = comparatorMask;
      break;
    }
    case _PWM_1: {
      *(pwm_base + _PWM_OUT_TOGGLE_EN_1_OFFSET) = comparatorMask;
      break;
    }
    case _PWM_2: {
      *(pwm_base + _PWM_OUT_TOGGLE_EN_2_OFFSET) = comparatorMask;
      break;
    }
    case _PWM_3: {
      *(pwm_base + _PWM_OUT_TOGGLE_EN_3_OFFSET) = comparatorMask;
      break;
    }
    case _PWM_4: {
      *(pwm_base + _PWM_OUT_TOGGLE_EN_4_OFFSET) = comparatorMask;
      break;
    }
    case _PWM_5: {
      *(pwm_base + _PWM_OUT_TOGGLE_EN_5_OFFSET) = comparatorMask;
      break;
    }
    case _PWM_6: {
      *(pwm_base + _PWM_OUT_TOGGLE_EN_6_OFFSET) = comparatorMask;
      break;
    }
    case _PWM_7: {
      *(pwm_base + _PWM_OUT_TOGGLE_EN_7_OFFSET) = comparatorMask;
      break;
    }
    default : break;
  }
}

/*--------------------------------------------------------------------------------------------------
 Function resets device.

 Parameters:
 None
--------------------------------------------------------------------------------------------------*/
void PWM_SoftReset(unsigned char *pwm_base) {
  *(pwm_base + _PWM_CONTROL_OFFSET) |= CONTROL_PWM_SOFT_RESET;
  // Need to put the reset bit back to 0
  *(pwm_base + _PWM_CONTROL_OFFSET) &= ~CONTROL_PWM_SOFT_RESET;
}

/*--------------------------------------------------------------------------------------------------
 Function enables the PWM outputs.

 Parameters:
 None
--------------------------------------------------------------------------------------------------*/
void PWM_EnableOutput() {
  *(pwm_base + _PWM_CTRL_OFFSET) |= CTRL_PWM_EN;
}

/*--------------------------------------------------------------------------------------------------
 Function disables the PWM outputs.

 Parameters:
 None
--------------------------------------------------------------------------------------------------*/
void PWM_DisableOutput(unsigned char *pwm_base) {
  *(pwm_base + _PWM_CTRL_OFFSET) &= ~CTRL_PWM_EN;
}
/*--------------------------------------------------------------------------------------------------
 Function gets current volume.

 Parameters:
 None

 Returns current volume (0 = muted)
--------------------------------------------------------------------------------------------------*/
unsigned char PWM_GetVolume(unsigned char *pwm_base) {
  return *(pwm_base + _PWM_VOLUME_OFFSET);
}

/*--------------------------------------------------------------------------------------------------
 Function sets volume.

 Parameters:
 vol - volume (0 = muted)
--------------------------------------------------------------------------------------------------*/
void PWM_SetVolume(unsigned char vol, unsigned char *pwm_base) {
  *(pwm_base + _PWM_VOLUME_OFFSET) = vol;
}

/*--------------------------------------------------------------------------------------------------
 Function enables PWM interrupt to the system when the specified PWM operation has completed.

 Parameters:
 interruptMask - the interrupts to be enabled
--------------------------------------------------------------------------------------------------*/
void PWM_EnableInterrupt(unsigned char interruptMask, unsigned char *pwm_base) {
  // If PWM interrupt (not FIFO interrupt) selected, the interrupt enable bit in CTRL register also needs to be set
  if (interruptMask & PWM_INT) {
    *(pwm_base + _PWM_CTRL_OFFSET) |= CTRL_PWM_INIT_IMASK;
  }
  *(pwm_base + _PWM_INT_MASK_OFFSET) |= interruptMask;
}

/*--------------------------------------------------------------------------------------------------
 Function disables PWM interrupt.

 Parameters:
 interruptMask - the interrupts to be disabled
--------------------------------------------------------------------------------------------------*/
void PWM_DisableInterrupts(unsigned char interruptMask, unsigned char *pwm_base) {
  // If PWM interrupt (not FIFO interrupt) selected, the interrupt enable bit in CTRL register also needs to be cleared
  if (interruptMask & PWM_INT) {
    *(pwm_base + _PWM_CTRL_OFFSET) &= ~CTRL_PWM_INIT_IMASK;
  }
  *(pwm_base + _PWM_INT_MASK_OFFSET) &= ~interruptMask;
}

/*--------------------------------------------------------------------------------------------------
 Function checks for pending PWM interrupt.

 Parameters:
 None

 Returns the content of the interrupt status register
--------------------------------------------------------------------------------------------------*/
unsigned char PWM_CheckPendingInterrupt(unsigned char *pwm_mask, unsigned char *pwm_base) {
  return *(pwm_base + _PWM_INT_STATUS_OFFSET);
}

/*--------------------------------------------------------------------------------------------------
 Function clears pending PWM interrupt flags.

 Parameters:
 None
--------------------------------------------------------------------------------------------------*/
void PWM_ClearPendingInterrupt(unsigned char *pwm_mask, unsigned char *pwm_base) {
  unsigned char regVal;
  //read/write to clear -> RW1C type
  regVal = *(pwm_base + _PWM_INT_STATUS_OFFSET);
  *(pwm_base + _PWM_INT_STATUS_OFFSET) = regVal;
}