/*
 * Project name:
     USB_UART_Click (Simple usage of UART module library functions)
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Description:
     This code demonstrates how to use uart library routines. Upon receiving
     data via USB UART click, MCU immediately sends it back to the sender.
 * Test configuration:
     MCU:             FT900
                      http://www.ftdichip.com/Support/Documents/DataSheets/ICs/DS_FT900_1_2_3.pdf
     Dev. Board:      Clicker 2 for FT900
                      http://www.mikroe.com/ft90x/clicker-2/
     Oscillator:      CPU at full system clock, 100.000MHz
     Ext. modules:    USB UART click - ac:UART_Click
     SW:              mikroC PRO for FT90x
                      http://www.mikroe.com/mikroc/ft90x/
 * NOTES:
     - Place USB UART click board into mikroBUS socket 1.
     - Use USART Terminal for communication with controller.
 */

char uart_rd;

void main()
{
  UART1_Init(56000);              // Initialize UART module at 56000 bps
  Delay_ms(100);                  // Wait for UART module to stabilize

  UART_Write_Text("Start");
  UART_Write(13);
  UART_Write(10);

  while (1)                       // Endless loop
  {
    if ( UART_Data_Ready() )
    {                             // If data is received
      uart_rd = UART_Read();     // read the received data
      UART_Write(uart_rd);       // and send data via UART
    }
  }
}